package blusci;

import processing.core.*; 
import fullscreen.*;  //um im Vollbildmodus starten zu können.

import codeanticode.gsvideo.*;
import processing.serial.*;

import org.scilab.modules.javasci.Scilab;
import org.scilab.modules.types.ScilabType;
import org.scilab.modules.types.ScilabDouble;

import kiba.hilfsklassen.Dateioperationen;

import kiba.betriebssystem.Linuxsystem;

public class BLUSCI4regler extends PApplet
{
    Scilab sci;
    
    int zaehler = 0; //für die Animation des roten Balls
    
    Dateioperationen dateioperationen;
    Linuxsystem linuxsystem;
    GSCapture webcam;
    PImage    bild;
    
    Serial myPort;  // Create object from Serial class    
    
    public void sprich(String text)
    {
        if(linuxsystem!=null)
            linuxsystem.syst("espeak -s100 -k20 -v mb-de7 \""+text+"\"");
        System.out.println(text);            
    }
    
    public void setup()
    {
        dateioperationen = new Dateioperationen();
        linuxsystem      = new Linuxsystem();
        try
        {
            sci = new Scilab();
            sci.open();                                     
        }
        catch(Exception e)
        {
            sprich("Problem beim Öffnen von Scilab:"+e);
            System.exit(0);
        }

        try
        {
//          webcam = new GSCapture(this,640,480,"/dev/video0");
          webcam = new GSCapture(this,640,480);
          webcam.start();
          bild = new PImage(640,480);
        }
        catch(Exception ee)
        {
            sprich("Problem beim Start der Kamera:"+ee);
            System.exit(0);
        }

        
        //Serielle Verbindung vorbereiten:
        linuxsystem.syst("ln -s  /dev/ttyACM0 /dev/ttyS80"); //wird nur gemacht, wenn noch nicht da.
        linuxsystem.syst("rm -f /var/lock/LCK..ttyS80");
        
        try
        {
            myPort = new Serial(this,"/dev/ttyS80", 9600);                
        }
        catch(Exception eee)
        {
            sprich("Problem beim Einrichten der seriellen Verbindung:"+eee);
            System.exit(0);
        }
        
//        sprich("Animation des grünen Balls, die Kamera und der Motorregler werden gestartet.");
        size(displayWidth, displayHeight);
        frameRate(20);
        (new FullScreen(this)).enter();         
    }
    
    public void draw()
    {                
        clear();
        fill(0);
        rect(0,0,displayWidth,displayHeight);
        fill(0,255,0);
        noStroke();
        if(zaehler<displayWidth/4)
            ellipse(displayWidth/4 + displayWidth/8 + zaehler,300,40,40);
        else
            ellipse(displayWidth/4 + displayWidth/8  + displayWidth/2 - zaehler,300,40,40);

            
        if(webcam!=null && webcam.available())
        {
            webcam.read();
            webcam.loadPixels();
            for(int i=0;i<webcam.pixels.length;i++)
                 bild.pixels[webcam.pixels.length-1-i]=webcam.pixels[i];
            bild.updatePixels();            
            set(displayWidth/2-640/2,displayHeight/2,bild);
            
            //Aktuelle Regelabweichung bestimmen:
            //1. Schwerpunkt der roten Pixel in der oberen Bildhälfte von Bild bestimmen:
            float xg = 0.0f; //gewichtete Position aufsummiert
            float x = 0.0f;  //horizontale Schwerpunktkoordinate
            float g = 0.0f;  //Gewichte aufsummiert
            bild.loadPixels();
            for(int i=0;i<bild.pixels.length/2;i++)
            {
                 float rot   = red(bild.get(i%640,i/640));
                 float gruen = green(bild.get(i%640,i/640));
                 float blau  = blue(bild.get(i%640,i/640));
                 float ROT   = (rot-(gruen+blau)*0.5f);
                 float BLAU   = (blau-(rot+gruen)*0.5f);
                 float GRUEN   = (gruen-(rot+blau)*0.5f);
                 if(GRUEN<0.0f)
                     GRUEN=0.0f;
                 g+=GRUEN;
                 xg+=GRUEN*(float)(i%640);                 
            }
            if(g>0.0f)
                x = xg/g;
            else
                x = 320.0f;
            //Schwerpunkt als senkrechte gelbe Linie im Bild einzeichnen:
            stroke(255,255,0);
            line(x+displayWidth/2-640/2,displayHeight/2,x+displayWidth/2-640/2,displayHeight/2+480);

            //Mitte weiss einzeichnen:
            line(320+displayWidth/2-640/2,displayHeight/2,320+displayWidth/2-640/2,displayHeight/2+480);
            
            //Einfachen Proportionalregler implementieren:            
            if ( myPort!=null && myPort.available() >= 1) 
            {
                int PWM = (int)(0.035f*(320.0f - x));
                if(PWM>20)
                    PWM=20;
                if(PWM<-20)
                    PWM=-20;
                    
                PWM = -PWM;    
                    
                myPort.write(PWM+20);
                
                text("PWM="+PWM,20,400);
                text("REGELABWEICHUNG="+(320.0f-x),20,430);
            }            
        }
            
        zaehler+=8;
        zaehler%=displayWidth/2;
    }   
    
    public static void main(String[] passedArgs) 
    {
        String[] appletArgs = new String[] { "blusci.BLUSCI4regler" };
        if(passedArgs != null) 
        {
             PApplet.main(concat(appletArgs, passedArgs));
        } 
        else 
        {
            PApplet.main(appletArgs);
        }            
    }    
}
