//Regler mit Beobachter mittels Scilab als SIL.

package blusci;

import processing.core.*; 
import fullscreen.*;  //um im Vollbildmodus starten zu können.

import codeanticode.gsvideo.*;
import processing.serial.*;

import org.scilab.modules.javasci.Scilab;
import org.scilab.modules.types.ScilabType;
import org.scilab.modules.types.ScilabDouble;

import kiba.hilfsklassen.Dateioperationen;

import kiba.betriebssystem.Linuxsystem;

public class BLUSCI8SCILAB extends PApplet
{
    //Für Scilab:
    long TALT,TNEU; //Systemzeit in Millisekunden für Scilabaufruf
    //letzter Systemzustand aus vorangehender Simulation:
    double[] YALT = {0.0,0.0,0.0,0.0};
    
    
    boolean ROTATE = false;
    
    Scilab sci;
    
    int zaehler = 0; //für die Animation des roten Balls
    
    Dateioperationen dateioperationen;
    Linuxsystem linuxsystem;
    GSCapture webcam;
    PImage    bild;
    
    Serial myPort;  // Create object from Serial class    
    
    public void sprich(String text)
    {
        if(linuxsystem!=null)
            linuxsystem.syst("espeak -s100 -k20 -v mb-de7 \""+text+"\"");
        System.out.println(text);            
    }
    
    public void setup()
    {
        dateioperationen = new Dateioperationen();
        linuxsystem      = new Linuxsystem();
        try
        {
            sci = new Scilab();
            sci.open();                                     
            
            String[] skriptzeilen = dateioperationen.ladeZeilen("initialisiere_beobachter.sci");
            //Initialisierungsscript laden und ausführen:
            for(int i=0;i<skriptzeilen.length;i++)
                sci.exec(skriptzeilen[i]);
            
        }
        catch(Exception e)
        {
            sprich("Problem beim Öffnen von Scilab:"+e);
            System.exit(0);
        }

        try
        {
//          webcam = new GSCapture(this,640,480,"/dev/video0");
          webcam = new GSCapture(this,640,480);
          webcam.start();
          bild = new PImage(640,480);
        }
        catch(Exception ee)
        {
            sprich("Problem beim Start der Kamera:"+ee);
            System.exit(0);
        }

        
        //Serielle Verbindung vorbereiten:
        linuxsystem.syst("ln -s  /dev/ttyACM0 /dev/ttyS80"); //wird nur gemacht, wenn noch nicht da.
        linuxsystem.syst("rm -f /var/lock/LCK..ttyS80");
        
        try
        {
            myPort = new Serial(this,"/dev/ttyS80", 9600);                
        }
        catch(Exception eee)
        {
            sprich("Problem beim Einrichten der seriellen Verbindung:"+eee);
            System.exit(0);
        }
        
//        sprich("Animation des grünen Balls, die Kamera und der Motorregler werden gestartet.");
        size(displayWidth, displayHeight);
        
        TALT=System.currentTimeMillis();
        TNEU=System.currentTimeMillis();
        
        frameRate(20);
        (new FullScreen(this)).enter();         
    }

    public void mouseClicked()
    {
        ROTATE = !ROTATE;
    }
    
    public void draw()
    {           
        int PWM=0;

        TALT=TNEU;
        TNEU=System.currentTimeMillis();
        
        clear();
        fill(0);
        rect(0,0,displayWidth,displayHeight);
        fill(0,255,0);
        noStroke();
        if(zaehler<displayWidth/4)
            ellipse(displayWidth/4 + displayWidth/8 + zaehler,160,40,40);
        else
            ellipse(displayWidth/4 + displayWidth/8  + displayWidth/2 - zaehler,160,40,40);

            
        if(webcam!=null && webcam.available())
        {
            webcam.read();
            webcam.loadPixels();
            for(int i=0;i<webcam.pixels.length;i++)
                 bild.pixels[webcam.pixels.length-1-i]=webcam.pixels[i];
            bild.updatePixels();            
            set(displayWidth/2-640/2,displayHeight/2,bild);
            
            //Aktuelle Regelabweichung bestimmen:
            //1. Schwerpunkt der roten Pixel in der oberen Bildhälfte von Bild bestimmen:
            float xg = 0.0f; //gewichtete Position aufsummiert
            float x = 0.0f;  //horizontale Schwerpunktkoordinate
            float g = 0.0f;  //Gewichte aufsummiert
            bild.loadPixels();
            for(int i=0;i<bild.pixels.length/8;i++)
            {
                 float rot   = red(bild.get(i%640,i/640));
                 float gruen = green(bild.get(i%640,i/640));
                 float blau  = blue(bild.get(i%640,i/640));
                 float ROT   = (rot-(gruen+blau)*0.5f);
                 float BLAU   = (blau-(rot+gruen)*0.5f);
                 float GRUEN   = (gruen-(rot+blau)*0.5f);
                 if(GRUEN<0.0f)
                     GRUEN=0.0f;
                 g+=GRUEN;
                 xg+=GRUEN*(float)(i%640);                 
            }
            if(g>0.0f)
                x = xg/g;
            else
                x = 320.0f;
            //Schwerpunkt als senkrechte gelbe Linie im Bild einzeichnen:
            stroke(255,255,0);
            line(x+displayWidth/2-640/2,displayHeight/2,x+displayWidth/2-640/2,displayHeight/2+480);

            //Mitte weiss einzeichnen:
            line(320+displayWidth/2-640/2,displayHeight/2,320+displayWidth/2-640/2,displayHeight/2+480);
            
            //Einfachen Proportionalregler implementieren:            
            if ( myPort!=null && myPort.available() >= 1) 
            {
                float FPWM = (0.025f*(320.0f - x));
                
                if(FPWM>0.0f)
                {
                    FPWM+=1.0f;
                    PWM = (int)(FPWM+0.5f);
                }                    
                else if(FPWM<0.0f)
                {
                    FPWM-=1.0f;
                    PWM = (int)(FPWM-0.5f);
                }
                    
                if(PWM>20)
                    PWM=20;
                if(PWM<-20)
                    PWM=-20;

                //SCILAB-BEREICH: statt P-Regler, Zustandsregler mit Beobachter
                //INPUT für Scilab vorbereiten:
                double real_phi = x-320.0;
                double dauer    = 0.001*(double)(TNEU-TALT);

                /*
                sci.exec("clear();");
                sci.exec("R=[0.1028838, 0.0075232, 0.0001731, 0.4205213];");
                sci.exec("a=36.0;");
                sci.exec("b=326.0;");
                sci.exec("c=115509.0;");
                sci.exec("real_phi=0.0;");
                sci.exec("function f = rechteSeite(t,y); sim_phi=y(1,1);sim_omega=y(2,1);sim_alfa=y(3,1);sim_intphi=y(4,1); u = -R*y; usim = u + 0.01*(real_phi-sim_phi); f(1,1) = sim_omega;f(2,1)=sim_alfa;f(3,1)=-a*sim_alfa-b*sim_omega+c*usim;f(4,1)=sim_phi;endfunction");
                */
                
                sci.exec("EINGABE=["+real_phi+","+dauer+","+YALT[0]+","+YALT[1]+","+YALT[2]+","+YALT[3]+"];");
                sci.exec("real_phi = EINGABE(1,1);");
                sci.exec("dauer    = EINGABE(1,2);");
                sci.exec("y0       = EINGABE(1,3:6)\';");
                sci.exec("t=linspace(0,dauer,10);");
                sci.exec("t0=0.0;");
                sci.exec("y=ode(y0,t0,t,rechteSeite);");
                //sci.exec("y=rechteSeite(t0,y0);");
                sci.exec("anz=size(y);");
                sci.exec("yout=y(:,anz(1,2))\';");
                sci.exec("u=-R*y;");
                sci.exec("anz=size(u);");
                //Nicht Mittelwert, sondern letzten Wert nehmen:
                //sci.exec("uout=mean(u);");
                sci.exec("anz=size(u);");
                sci.exec("uout=u(anz(1,1),anz(1,2));");
                sci.exec("AUSGABE=[uout,yout];");
                try
                {
                    ScilabType ysci = sci.get("AUSGABE");
                    //ScilabType ysci = sci.get("y");
                    double[][] yscireal = ((ScilabDouble)ysci).getRealPart();
                    //text("yscireal zeilen="+yscireal.length+" spalten="+yscireal[0].length,20,460);
                    text("PWM Zustandsregler="+yscireal[0][0],20,460);
                    float SCIPWM=(float)yscireal[0][0];
                    YALT[0]=yscireal[0][1];
                    YALT[1]=yscireal[0][2];
                    YALT[2]=yscireal[0][3];
                    YALT[3]=yscireal[0][4];

                    if(SCIPWM>0.0f)
                    {
                        SCIPWM+=2.2f;
                        PWM = (int)(SCIPWM+0.5f);
                    }                    
                    else if(SCIPWM<0.0f)
                    {
                        SCIPWM-=2.2f;
                        PWM = (int)(SCIPWM-0.5f);
                    }
                    
                    if(PWM>20)
                        PWM=20;
                    if(PWM<-20)
                        PWM=-20;
                    
                }
                catch(Exception eee)
                {
                    YALT[0]=0.0;
                    YALT[1]=0.0;
                    YALT[2]=0.0;
                    YALT[3]=0.0;
                    text("SCIERR="+eee,20,460);
                }
                    
                    
                PWM = -PWM;    
                
                if(ROTATE)
                    PWM=4;
                myPort.write(PWM+20);
           
                
                
                text("PWM="+PWM,20,400);
                text("REGELABWEICHUNG="+(320.0f-x),20,430);
            }            
        }
            
        zaehler+=2;
        zaehler%=displayWidth/2;
    }   
    
    public static void main(String[] passedArgs) 
    {
        String[] appletArgs = new String[] { "blusci.BLUSCI8SCILAB" };
        if(passedArgs != null) 
        {
             PApplet.main(concat(appletArgs, passedArgs));
        } 
        else 
        {
            PApplet.main(appletArgs);
        }            
    }    
}
