package kiba.betriebssystem;

import java.util.Random;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.Vector;
import java.util.ArrayList;

import kiba.hilfsklassen.Parser;

 /**
 *  <h4>Linuxsystem.java</h4>
 *
 *  Copyright (C) 2013 Guido Kramann<br/>
 *  kramann@fh-brandenburg.de<br/>
 *  http://www.kramann.info<br/>
 *  <br/>
 *  <p>  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  </p>
 *  <p>
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  </p>
 *  <p>
 *  You should have received a copy of the GNU General Public License
 *  along with this program (gpl.html).  If not, see http://www.gnu.org/licenses/.
 *  </p>
 *  ACHTUNG: String aufruf(String) führt aus bisher nicht gefundenen Gründen manchmal zu Systemabstürzen.<br/>
 *           Deshalb ist der Befehl void syst(String) vorzuziehen.
 *           
 *  ACHTUNG: Die main-Methode läßt sich nicht direkt aus bluej starten.         
 *           Siehe dazu das Skript teste_linuxsystem oben im Projektordner
 *           
 *  Zum Erstellen der .so-Datei siehe Ordner libraries/linuxsystem im Projektordner.         
 */
public final class Linuxsystem
{
    private Random zufall = new Random(System.currentTimeMillis());

    private Parser parser = new Parser();    

    /**
    Ruft einen Systembefehl auf und liefert dessen Rückgabe als String zurück.<br/><br/>
    Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath() <br/>
    liefert den absoluten Pfad des jar-Files, in dem sich Linuxsystem.class befindet.
    */
    public final native String aufrufen(String command);

    /**
    Einfacher Aufruf von "system(...)" von der eingebetteten C-Funktion aus.
    */
    public final native void syst(String command);

    public void systThread(String command)
    {
       SystThread st = new SystThread(command);
       (new Thread(st)).start();
    }

    /**Hilfsklasse zum Starten eines neuen Thzreads bei einem Aufruf.*/
    private final class SystThread implements Runnable
    {
         String bef;

         public SystThread(String command)
         {
             bef = command;
         }

         public void run()
         {
             syst(bef);
         }
    }


    public Linuxsystem()
    {
    }

    /**Nutzt public native void syst(String command), um 
       - das übergebene Kommando auszuführen,<br/>
       - dessen Ausgabe in eine Datei mit zufällig erzeugtem Namen zu speichern, <br/>
       - diese Datei auszulesen und den Inhalt zeilenweise zurückzuliefern, <br/>
       - die Datei wieder zu löschen.<br/>
       Hinweis: Es werden der Standard- und der Fehlerkanal in die Zieldatei geschrieben.
    */
    public final String[] call(String command)
    {
        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);

        long x = zufall.nextLong();
        
        pfad+="/.temporaer"+x;

        syst(command+" > "+pfad+" 2>&1");        

        String[] ergebnis = null;
        Vector<String> vektor;
        File datei = new File(pfad);
        if(datei.exists()==false)
            return null;

        try
        {
            vektor = new Vector<String>();

            FileReader meinLeser = new FileReader( new File(pfad) );
            BufferedReader pufferleser  = new BufferedReader(meinLeser);

            String zeile = pufferleser.readLine();
            while( zeile!=null )
            {
                vektor.add(zeile);
                zeile = pufferleser.readLine();
            }
            meinLeser.close();
            syst("rm -f "+pfad);
            ergebnis = (String[])vektor.toArray(new String[0]);
            return ergebnis;
        }
        catch(IOException e) 
        { 
            syst("rm -f "+pfad);
            return null;
        }
    }

    public final String callString(String command)
    {
        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
        
        pfad+="/.temporaer"+x;

        syst(command+" > "+pfad+" 2>&1");        

        StringBuffer ergebnis = new StringBuffer();
        File datei = new File(pfad);
        if(datei.exists()==false)
            return null;

        try
        {
            FileReader meinLeser = new FileReader( new File(pfad) );
            BufferedReader pufferleser  = new BufferedReader(meinLeser);

            int zeile = pufferleser.read();
            while( zeile!=-1 )
            {
                ergebnis.append((char)zeile);
                zeile = pufferleser.read();
            }
            meinLeser.close();
            syst("rm -f "+pfad);
            return ergebnis.toString();
        }
        catch(IOException e) 
        { 
            syst("rm -f "+pfad);
            return null;
        }        
    }

    public final byte[] callBytes(String command)
    {
        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
        
        pfad+="/.temporaer"+x;

        syst(command+" > "+pfad+" 2>&1");        

        try
        {
            File meinFile = new File(pfad);
            FileInputStream meinStream = new FileInputStream(meinFile);
            int anzahl = meinStream.available();
            byte[] daten = new byte[ anzahl ];            
            meinStream.read( daten );
            meinStream.close();
            syst("rm -f "+pfad);
            return daten;
        }
        catch(IOException e)
        {
            System.out.println("Fehler beim Laden:");
            System.out.println(e.getMessage());
            syst("rm -f "+pfad);
            return null;            
        }        
    }


    private final void speichern(String dateiname, String text)
    {
        try
        {
            File              meinFile = new File(dateiname);
            FileWriter   meinSchreiber = new FileWriter(meinFile);
            PrintWriter  meinPrinter   = new PrintWriter( meinSchreiber );            
            meinPrinter.print(text);            
            meinPrinter.close();   
        }
        catch(IOException e)
        {
            System.out.println(e.getMessage());
        }
    }

    public void speichernBinaer(String name, byte[] daten)
    {
        try
        {
            File meinFile = new File(name);
            FileOutputStream meinStream = new FileOutputStream(meinFile);            
            meinStream.write(daten);
            meinStream.close();
        }
        catch(IOException e)
        {
            System.out.println("Fehler beim Speichern:");
            System.out.println(e.getMessage());
        }    
    }

    private final String holeZufallspfad()
    {
        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);


        long x = zufall.nextLong();
        
        pfad+="/.tmpz"+x;
        return pfad;
    }

    public final String encodeString(String text)
    {
        String datei = holeZufallspfad();
        //speichern(datei,text);

        byte[] bbb=new byte[]{};
        try
        {
            bbb=text.getBytes("UTF-8");
        }
        catch(Exception eeeeee)
        {
            System.out.println(eeeeee);
        }

        speichernBinaer(datei,bbb);
        //String ergebnis = callString("base64 -w 0 "+datei+" | encode ");
        byte[] ergebnis = callBytes("base64 -w 0 "+datei+" | encode ");
        syst("rm -f "+datei);
        String eee="";
        try
        {
            eee=new String(ergebnis,"UTF-8");
        }
        catch(Exception eeeee)
        {
            System.out.println(eeeee);
        }
        return eee;
    }

    public final String decodeString(String text)
    {
        String datei = holeZufallspfad();
        String datei2 = datei+"2";
        //speichern(datei,text);

        byte[] bbb=new byte[]{};
        try
        {
            bbb=text.getBytes("UTF-8");
        }
        catch(Exception eeeeee)
        {
            System.out.println(eeeeee);
        }

        speichernBinaer(datei,bbb);
        syst("cat "+datei+" | decode > "+datei2);
        syst("rm -f "+datei);
        byte[] ergebnis = callBytes("base64 -d -w 0 "+datei2);
        syst("rm -f "+datei2);
        String eee="";
        try
        {
            eee=new String(ergebnis,"UTF-8");
        }
        catch(Exception eeeee)
        {
            System.out.println(eeeee);
        }
        return eee;
    }

    public final String encodeBytes(byte[] bytes)
    {
        String datei = holeZufallspfad();
        speichernBinaer(datei,bytes);
        String ergebnis = callString("base64 -w 0 "+datei+" | encode ");
        syst("rm -f "+datei);
        return ergebnis;        
    }

    public final byte[] decodeBytes(String text)
    {
        String datei = holeZufallspfad();
        String datei2 = datei+"2";
        speichern(datei,text);
        syst("cat "+datei+" | decode > "+datei2);
        syst("rm -f "+datei);
        byte[] ergebnis = callBytes("base64 -d -w 0 "+datei2);
        syst("rm -f "+datei2);
        return ergebnis;
    }



    static
    {
        String pfad = Linuxsystem.class.getProtectionDomain().getCodeSource().getLocation().getPath();  
System.out.println("Pfad="+pfad);
        //Jar-File-Name entfernen:
        int i = pfad.length()-1;
        while(pfad.charAt(i)!='/')
            i--;
        pfad = pfad.substring(0,i);

        //Dynamische Bibliothek laden:
        System.load(pfad+"/aufrufen.so");
    }   

    public static void main(String[] args)
    {
        Linuxsystem ls = new Linuxsystem();


        System.out.println("Testen der Funktion String aufruf(String):");
        System.out.println(ls.aufrufen("ls"));
        System.out.println("Testen der Funktion void syst(String):");
        ls.syst("echo \"huhu\" | leafpad");
        ls.syst("echo \"huhu (Dieser Aufruf erfolgte mit system() in C)\"");

/*        
        System.out.println("\n\nTesten von String call(String) mit \"javac huhu.java\":");
        String[] ergebnis = ls.call("javac huhu.java");
        for(int i=0;i<ergebnis.length;i++) 
            System.out.println(ergebnis[i]);
*/

        System.out.println("\n\nTeste von encodeString und decodeString mit \"Dies ist ein Satz.\"");        
        String erg = ls.encodeString("Dies ist ein Satz.");
        System.out.println("encoded:"+erg);
        String decoded = ls.decodeString(erg);
        System.out.println("decoded:"+decoded);


        System.out.println("\n\nTeste von encodeBytes und decodeBytes mit {65,66,67}");        
        String er = ls.encodeBytes(new byte[] {65,66,67});
        System.out.println("encoded:"+er);
        byte[] de = ls.decodeBytes(er);
        System.out.println("decoded:"+(int)de[0]+","+(int)de[1]+","+(int)de[2]);

        System.out.println("PRÜFEN, OB DECODE/ENCODE AUCH MIT UMLAUTEN FUNKTIONIERT:");

        System.out.println("\n\nTeste von encodeString und decodeString mit \"Daß über mäandernden Mühlenrädern...\"");        
        erg = ls.encodeString("Daß über mäandernden Mühlenrädern...");
        System.out.println("encoded:"+erg);
        decoded = ls.decodeString(erg);
        System.out.println("decoded:"+decoded);


    }
}
