package kiba.hilfsklassen;

import java.util.Random;

import kiba.betriebssystem.Linuxsystem;
import kiba.hilfsklassen.Dateioperationen;

/**
 * Methoden zum Filesystem
 */
public class Filesystem
{
    Linuxsystem linuxsystem = new Linuxsystem();
    Dateioperationen dateioperationen = new Dateioperationen();    
    Random zufall = new Random(System.currentTimeMillis());

    /**
     * In Datei- und Ordnernamen können Parameter enthalten sein.
     * Die Art und Weise, wie das umgesetzt werden soll, soll möglichst
     * ohne komplizierte Konventionen funktionieren:
     * 
     * Ein Unterstrich zeigt einen Parameter an, dessen Name dann folgt.
     * Unmittelbar darauf kommt eine positive Integer-Zahl, die die Parameterbelegung
     * repräsentiert.
     * 
     * Soll beispielsweise das Tempo als Parameter TEMPO mit dem Wert 88 (bpm)
     * in einen Ordnenamen integriert werden, so kann dies beispielsweise so
     * geschehen:
     * 
     * ordnername_TEMPO88_WEITEREPARAMTER2
     * 
     * Die nachfolgende Methode versucht aus einem Datei- oder Ordnernamen
     * die Belegung eines als Bezeichner übergebenen Parameters zu finden und liefert
     * -1, wenn dieser nicht existiert.
     * 
     */
     public int holeParameter(String dateiname, String parameterbezeichner)
     {
          if(dateiname==null || parameterbezeichner==null || dateiname.length()<parameterbezeichner.length()+2)         
              return -1;
              
          for(int i=0;i<dateiname.length()-parameterbezeichner.length()-2;i++)
          {
               if(dateiname.substring(i,i+parameterbezeichner.length()+1).equals("_"+parameterbezeichner))
               {
//System.out.println(parameterbezeichner+" gefunden!");                   
                   //Anzahl der Zeichen des Integerwertes ermitteln:
                   int anz=0;
                   int pos = i+parameterbezeichner.length()+1;
                   while(pos+anz<dateiname.length() && (int)dateiname.charAt(pos+anz)>=48 && 
                                                        (int)dateiname.charAt(pos+anz)<=57 )
                       anz++;
                   if(anz>0)    
                   {
                       try
                       {
                           int wert = (int)Double.parseDouble(dateiname.substring(pos,pos+anz));
                           return wert;
                       }
                       catch(Exception e)
                       {          
                           //System.out.println(e);
                       }
                   }
               }
          }
          return -1;
     }
    
    public String[] holeVerzeichnisinhalt(String pfad)
    {
        return linuxsystem.call("cd "+pfad+";ls");
    }
    public String[] holeOrdnerInVerzeichnis(String pfad)
    {
        String[] erg =  holeVerzeichnisinhalt(pfad);
        int anz=0;
        for(int i=0;i<erg.length;i++)
            if(dateioperationen.pruefeObPfadVerzeichnisIst(pfad+"/"+erg[i]))
                anz++;
        String[] ordner = new String[anz];                
        anz=0;
        for(int i=0;i<erg.length;i++)
        {
            if(dateioperationen.pruefeObPfadVerzeichnisIst(pfad+"/"+erg[i]))
            {
                ordner[anz]=erg[i];
                anz++;
            }
        }            
        return ordner;
    }
    public String[] holeDateienInVerzeichnis(String pfad)
    {
        String[] erg =  holeVerzeichnisinhalt(pfad);
        int anz=0;
        for(int i=0;i<erg.length;i++)
            if(!dateioperationen.pruefeObPfadVerzeichnisIst(pfad+"/"+erg[i]))
                anz++;
        String[] ordner = new String[anz];                
        anz=0;
        for(int i=0;i<erg.length;i++)
        {
            if(!dateioperationen.pruefeObPfadVerzeichnisIst(pfad+"/"+erg[i]))
            {
                ordner[anz]=erg[i];
                anz++;
            }
        }            
        return ordner;
    }
    
    public String holeZufallsordnerAusVerzeichnis(String pfad)    
    {
         String[] ordner = holeOrdnerInVerzeichnis(pfad);
         if(ordner==null || ordner.length==0)
             return null;
             
         return ordner[zufall.nextInt(ordner.length)];             
    }
    
    public String holeZufallsdateiAusVerzeichnis(String pfad)    
    {
         String[] ordner = holeDateienInVerzeichnis(pfad);
         if(ordner==null || ordner.length==0)
             return null;
             
         return ordner[zufall.nextInt(ordner.length)];             
    }
    
    public void kopiereDatei(String quellpfad, String name, String zielpfad, String neuername)
    {
        loescheDateiInVerzeichnis(zielpfad,neuername);
        linuxsystem.call("cp -f "+quellpfad+"/"+name+" "+zielpfad+"/"+neuername);
    }
    
    public void kopiereDatei(String quellpfad, String name, String zielpfad)
    {
        loescheDateiInVerzeichnis(zielpfad,name);
        linuxsystem.call("cp -f "+quellpfad+"/"+name+" "+zielpfad);
    }
    
    public void verschiebeDatei(String quellpfad, String name, String zielpfad)
    {
        loescheDateiInVerzeichnis(zielpfad,name);
        linuxsystem.call("mv -f "+quellpfad+"/"+name+" "+zielpfad);
    }
    
    public void kopiereOrdner(String quellpfad, String name, String zielpfad)
    {
        loescheOrdnerInVerzeichnis(zielpfad,name);
        linuxsystem.call("cp -r -f "+quellpfad+"/"+name+" "+zielpfad);
    }
    
    public void verschiebeOrdner(String quellpfad, String name, String zielpfad)
    {
        loescheOrdnerInVerzeichnis(zielpfad,name);
        linuxsystem.call("mv -f "+quellpfad+"/"+name+" "+zielpfad);
    }
    
    public void loescheDateiInVerzeichnis(String pfad, String name)
    {
        linuxsystem.call("rm -f "+pfad+"/"+name);
    }
    
    public void loescheOrdnerInVerzeichnis(String pfad, String name)
    {
        linuxsystem.call("rm -r -f "+pfad+"/"+name);
    }
    
    public void erstelleOrdnerInVerzeichnis(String pfad, String name)
    {
        loescheOrdnerInVerzeichnis(pfad,name);
        linuxsystem.call("mkdir "+pfad+"/"+name);
    }
    
    public boolean verzeichnisEnthaeltOrdner(String pfad)
    {
        String[] erg = holeOrdnerInVerzeichnis(pfad);
        if(erg==null || erg.length==0)
            return false;
        else
            return true;
    }
    
    public static void main(String[] args)
    {
        String pfad = "/mnt-system";
        Filesystem fs = new Filesystem();
        //String[] erg = fs.holeVerzeichnisinhalt("/mnt-system");
        //String[] erg = fs.holeOrdnerInVerzeichnis("/mnt-system");
        String[] erg = fs.holeDateienInVerzeichnis(pfad);
        for(int i=0;i<erg.length;i++)
        {
            System.out.println("Nr."+i+": "+erg[i]);
        }
        
        System.out.println("Zufallsdatei:");
        System.out.println(fs.holeZufallsdateiAusVerzeichnis(pfad));
        System.out.println("Zufallsordner:");
        System.out.println(fs.holeZufallsordnerAusVerzeichnis(pfad));
        
        System.out.println("Beispiel dafür einen Parameter auszulesen.");
        String testname = "ordnername_TEMPO88_TEMPOZWEI120";
        System.out.println("Test mit:"+testname);
        System.out.println("TEMPO=="+fs.holeParameter(testname,"TEMPO"));
        System.out.println("TEMPOZWEI=="+fs.holeParameter(testname,"TEMPOZWEI"));
        
    }
}
